from dataclasses import dataclass, field
from enum import Enum
from typing import Dict, List
from datetime import datetime

class InstagramMediaType(Enum):
    IMAGE = 1
    VIDEO = 2
    CAROUSEL_IMAGE = 8

@dataclass
class InstagramDataclassInterface:
    _data: Dict = field(default_factory=dict, repr=False)
    
    def __post_init__(self):
        if len(self._data) > 0:
            data = {k.lower(): v for k, v in self._data.items() if k.lower() in self.__dict__.keys()}
            self.__dict__.update(data)
    
    def to_dict(self):
        return {k: v for k, v in self.__dict__.items() if k != "_data"}

@dataclass
class InstagramUser(InstagramDataclassInterface):
    pk: int = field(default=None)
    username: str = field(default=None)
    full_name: str = field(default=None)
    is_private: bool = field(default=None)
    profile_pic_url: str = field(default=None)

@dataclass
class InstagramCaption(InstagramDataclassInterface):
    pk: int = field(default=None)
    user: InstagramUser = field(default=None)
    text: str = field(default=None)
    type: int = field(default=None)
    created_at: datetime = field(default=None)
    content_type: str = field(default=None)
    status: str = field(default=None)

    def __post_init__(self):
        super().__post_init__()
        self.user = InstagramUser(self.user)

@dataclass
class InstagramImage(InstagramDataclassInterface):
    width: int = field(default=None)
    height: int = field(default=None)
    url: str = field(default=None)

@dataclass
class InstagramContent(InstagramDataclassInterface):
    pk: int = field(default=None)
    id: int = field(default=None)
    taken_at: datetime = field(default=None)
    media_type: InstagramMediaType = field(default=None)
    code: str = field(default=None)
    comment_count: int = field(default=None)
    user: InstagramUser = field(default=None)
    like_count: int = field(default=None)
    caption: InstagramCaption = field(default=None)
    accessibility_caption: str = field(default=None)
    original_width: int = field(default=None)
    original_height: int = field(default=None)
    images: List[InstagramImage] = field(default_factory=list)
    
    def __post_init__(self):
        super().__post_init__()
        self.media_type = InstagramMediaType(self.media_type)
        self.user = InstagramUser(self.user)
        self.caption = InstagramCaption(self.caption)
        if "image_versions2" in self._data:
            self.images = [InstagramImage(image) for image in self._data['image_versions2']['candidates']]
        elif "carousel_media" in self._data:
            self.images = []
            for media in self._data['carousel_media']:
                self.images += [InstagramImage(image) for image in media['image_versions2']['candidates']]



import os
import time
import json
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.ui import WebDriverWait
from .webdriver import WebDriver

COOKIE_PATH = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    "tmp",
    "cookies.json"
)

def load_cookies():
    if not os.path.exists(COOKIE_PATH):
        return None

    with open(COOKIE_PATH, "r") as file:
        cookies = json.loads(file.read())
    return cookies

def save_cookies(cookies):
    os.makedirs(os.path.dirname(COOKIE_PATH), exist_ok=True)
    with open(COOKIE_PATH, "w") as file:
        json.dump(cookies, file)

def signin(username: str, password: str, use_cookie: bool=True):
    loaded_cookies = load_cookies() if use_cookie else None

    driver = WebDriver()
    wait = WebDriverWait(driver, 15, 500)

    driver.get("https://www.instagram.com")
    driver.implicitly_wait(5)

    if loaded_cookies is None:
        # Enter usernae
        username_input = wait.until(expected_conditions.visibility_of_element_located((By.NAME, "username")))
        time.sleep(1)
        WebDriver.send_keys(username_input, username)

        # Enter password
        password_input = wait.until(expected_conditions.visibility_of_element_located((By.NAME, "password")))
        time.sleep(1)
        WebDriver.send_keys(password_input, password)

        # Click sign-in button
        signin_button = wait.until(expected_conditions.visibility_of_element_located((By.CSS_SELECTOR, "#loginForm > div > div:nth-child(3) > button")))
        signin_button.click()
        time.sleep(1)
    else:
        for cookie in loaded_cookies:
            driver.add_cookie(cookie)
        print("Saved cookies have been loaded.")

    # Search any tag to build more cookies
    driver.get("https://www.instagram.com/explore/tags/instagram")
    driver.implicitly_wait(5)

    register_button_ko, register_button_en = None, None
    try:
        register_button_ko = driver.find_element(By.XPATH, "//div[text()=\"가입하기\"]")
    except:
        pass
    try:
        register_button_en = driver.find_element(By.XPATH, "//div[text()=\"Sign Up\"]")
    except:
        pass
    if register_button_ko is not None or register_button_en is not None:
        # Not signed in
        return signin(username, password, False)

    # Get activated cookies after sign-in
    cookies = driver.get_cookies()

    driver.quit()

    if loaded_cookies is None or not use_cookie:
        save_cookies(cookies)
        print(f"Cookies have been saved in `{COOKIE_PATH}`.")

    return cookies



import os
import time
from selenium import webdriver

class WebDriver(webdriver.Chrome):
    def __init__(self):
        options = webdriver.ChromeOptions()
        options.add_argument("headless")
        options.add_argument("no-sandbox")
        options.add_argument("incognito")
        options.add_argument("lang=ko")
        options.add_argument("disable-dev-shm-usage")
        options.add_argument("disable-gpu")
        options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36")
        options.page_load_strategy = 'normal'

        driver_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__ + "/../")),
            "drivers",
            "chromedriver",
        )
        self.driver = super().__init__(driver_path, options=options)

    def send_keys(element, sequence: str):
        for character in sequence:
            element.send_keys(character)
            time.sleep(0.5)



import json
import pandas as pd
import traceback
import requests
from .signin import signin
from .dto import *

class InstagramParser:
    def __init__(self, username, password, keyword, extract_all=False):
        self.username = username
        self.password = password
        self.keyword = keyword
        self.extract_all = extract_all
        self.cookies = None
        self.contents = None
        self.headers = {
            "accept-encoding": "gzip, deflate, br",
            "accept-language": "ko,en;q=0.9,en-US;q=0.8",
            "access-control-expose-headers": "X-IG-Set-WWW-Claim",
            "x-ig-app-id": "936619743392459",
        }

    def run(self, output_file=None):
        media_list = []

        # Sign-in and load cookies
        cookies = signin(self.username, self.password)
        self.cookies = {cookie['name']: cookie['value'] for cookie in cookies}
        self.headers['x-csrftoken'] = self.cookies['csrftoken']
        print("Signed in.")

        # Request web_info, which is search result summary information
        print("Loading `web_info`...")
        web_info = self.request_web_info()
        media_list += self.parse_sections(web_info['top'])
        media_list += self.parse_sections(web_info['recent'])
        print("`web_info` loaded.")

        page, max_id = 1, ""
        print(f"Loading `section_info`...")
        try:
            while True:
                section_info = self.request_section_info(page, max_id)
                media_list += self.parse_sections(section_info)
                print(f"`section_info`[{page}] loaded.")

                if section_info['more_available']:
                    page = section_info['next_page']
                    max_id = section_info['next_max_id']
                else:
                    break
        except Exception:
            traceback.print_exc()
            pass
        print("All `section_info` loaded.")

        if output_file is not None:
            if output_file[-4:].lower() == ".csv":
                # Save as csv
                df = pd.DataFrame(media_list)
                df.to_csv(output_file)
            else:
                # Save as json
                with open(output_file, "w") as file:
                    string = json.dumps(media_list)
                    file.write(string)
            print(f"Data saved in `{output_file}`.")

        return media_list

    def request_web_info(self):
        response = requests.get(
            url="https://i.instagram.com/api/v1/tags/web_info/",
            params={"tag_name": self.keyword},
            headers=self.headers,
            cookies=self.cookies,
        )
        
        try:
            web_info = response.json()['data']
        except Exception:
            print(response.status_code, response.content)
            raise Exception("Failed to load `web_info`.")
        
        return web_info
    
    def request_section_info(self, page, max_id):
        response = requests.post(
            url=f"https://i.instagram.com/api/v1/tags/{self.keyword}/sections/",
            headers=self.headers,
            cookies=self.cookies,
            data={
                "page": page,
                "max_id": max_id,
                "include_persistent": 0,
                "surface": "grid",
                "tab": "recent",
            },
        )

        try:
            section_info = response.json()
        except Exception:
            print(response.status_code, response.content)
            raise Exception("Failed to load `section_info`.")
        
        return section_info

    def parse_sections(self, contents):
        contents = self.parse_sections_as_dict(contents)

        if self.extract_all:
            return contents
        
        contents_data = [InstagramContent(content).to_dict() for content in contents]

        return contents_data

    def parse_sections_as_dict(self, contents):
        section_medias = [
            content['layout_content']['medias']
            for content
            in contents['sections']
            if content['layout_type'] == "media_grid"
        ]

        contents_data = []
        for row_medias in section_medias:
            for media in row_medias:
                contents_data += media.values()

        return contents_data
    
    '''def parse_contents(self):
        prefixes = (
            "https://i.instagram.com/api/v1/tags/web_info/",
            f"https://i.instagram.com/api/v1/tags/{parse.quote(self.keyword)}/sections/",
        )
        contents = [
            json.loads(base64.b64decode(entry['response']['content']['text']).decode("utf-8"))
            for entry
            in self.data['entries']
            if entry['request']['method'] in ["GET", "POST"]
                and entry['request']['url'].startswith(prefixes)
                and 'text' in entry['response']['content'].keys()
        ]
        
        if len(contents) == 0:
            raise Exception("No instagram content was found in the input har file!")

        self.contents = contents
        return contents'''



import json
import pandas as pd
import traceback
import requests
import re
import datetime
from typing import Tuple
from bs4 import BeautifulSoup
#from .dto import *

class GooglePlayReviewParser:
    def __init__(self, app_id: str):
        self.app_id = app_id
        self.cookies = None
        self.headers = {
            "accept-encoding": "gzip, deflate, br",
        }
    
    def run(self, output_file: str = "") -> list:
        contents, scripts = self.request_app_details()
        review_ds_keys = self.find_review_ds_keys(contents)
        review_data = self.crawl_review_data(scripts, review_ds_keys)
        
        if output_file is not None and len(output_file) > 0:
            if output_file[-4:].lower() == ".csv":
                # Save as csv
                df = pd.DataFrame(review_data)
                df.to_csv(output_file)
            elif output_file[-5:].lower() == ".json":
                # Save as json
                with open(output_file, "w") as file:
                    string = json.dumps(review_data)
                    file.write(string)
            else:
                # Save as text
                with open(output_file, "w", encoding="UTF-8") as file:
                    string = str(review_data)
                    file.write(string)
            print(f"Data saved in `{output_file}`.")
        
        return review_data

    def request_app_details(self) -> Tuple[str, list]:
        response = requests.get(
            url="https://play.google.com/store/apps/details",
            params={
                "id": self.app_id,
                "hl": "ko",
            },
            headers=self.headers,
            cookies=self.cookies,
        )
        
        try:
            app_details = response.text
            scripts = BeautifulSoup(response.text, "html.parser").select("script")
        except Exception:
            print(response.status_code, response.content)
            raise Exception("Failed to load `app_details`.")
        
        return app_details, scripts

    def find_review_ds_keys(self, contents: str) -> list:
        # Find data service keys
        services_regex = r"var AF_dataServiceRequests = ([^;]*)"
        services_match = re.findall(services_regex, contents, re.MULTILINE)
        services = services_match[0]
        review_ds_key_regex = r"\'(ds:[0-9]+)\' : {id:\'oCPfdb\'"
        review_ds_keys = re.findall(review_ds_key_regex, services, re.MULTILINE)

        return review_ds_keys
    
    def parse_review(self, review_row: list):
        writed_time = datetime.datetime.utcfromtimestamp(review_row[5][0])
        writed_time = writed_time.strftime("%Y-%m-%d %H:%M")

        return {
            "review_id": review_row[0],
            "author": review_row[1][0],
            "review_text": review_row[4],
            "rating": review_row[2],
            "writed_time": writed_time,
        }
    
    def crawl_review_data(self, scripts: list, review_ds_keys: list) -> list:
        review_data, next_page = self.review_first_page(scripts, review_ds_keys)

        while len(next_page) > 0:
            review_data_partial, next_page = self.review_other_page(next_page)
            review_data += review_data_partial

        return review_data
    
    def review_first_page(self, scripts: list, review_ds_keys: list) -> Tuple[list, str]:
        review_data = []
        next_page = ""

        for script in scripts:
            ds_key_regex = r"AF_initDataCallback\({key: \'(ds:\d+)\',"
            ds_keys = re.findall(ds_key_regex, script.text, re.MULTILINE)
            if len(ds_keys) > 0 and ds_keys[0] in review_ds_keys:
                review_data_regex = r"data:([\[].*[\]]), sideChannel: {}}\);$"
                review_data_str = re.findall(review_data_regex, script.text, re.MULTILINE)[0]
                review_data_raw = json.loads(review_data_str)
                
                review_data = list(map(self.parse_review, review_data_raw[0]))
                next_page = str(review_data_raw[1][1])
                break
        
        return review_data, next_page

    def review_other_page(self, page_hash: str) -> Tuple[list, str]:
        review_data = []
        next_page = ""

        try:
            f_req_format = '[[["oCPfdb","[null,[2,1,[100,null,\\"%s\\"]],[\\"%s\\",7]]",null,"generic"]]]'
            response = requests.post(
                url="https://play.google.com/_/PlayStoreUi/data/batchexecute",
                params={
                    "rpcids": "oCPfdb",
                    "hl": "ko",
                },
                data={
                    "f.req": f_req_format % (page_hash, self.app_id)
                },
                headers=self.headers,
                cookies=self.cookies,
            )
            review_data_raw = json.loads(json.loads(response.text[6:])[0][2])

            review_data = list(map(self.parse_review, review_data_raw[0]))
            if review_data_raw[1] is None:
                print("Scrapped all reviews.")
            else:
                next_page = review_data_raw[1][1]
        except Exception:
            traceback.print_exc()
            pass

        return review_data, next_page



import os
import setuptools

def requirements():
    with open(os.path.join(os.path.dirname(__file__), "requirements.txt"), encoding="utf-8") as f:
        return f.read().splitlines()

setuptools.setup(
    name="default-scraper",
    version="1.1.1",
    license="MIT",
    author="Seongbum Seo",
    author_email="sbumseo@bigpicture.team",
    description="Web Scraper",
    long_description=open("README.md").read(),
    url="https://github.com/bigpicture-kr/default-scraper",
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_data={
        "default_scraper": ["drivers/chromedriver"],
    },
    zip_safe=True,
    install_requires=requirements(),
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
)
